import os
import re
import subprocess
from enum import Enum

import semver
import toml
import typer
from plumbum import FG, ProcessExecutionError
from plumbum import local as lr

app = typer.Typer()


def git(*args, output=True):
    r = subprocess.check_output(["git"] + list(args))
    if output:
        print(args)  # noqa
        print(r)  # noqa
    return r


def git_num_changes():
    return len(git("status", "--porcelain").splitlines())


def _get_version():
    return git("describe", "--tags").decode().strip()


@app.command()
def push_repo():
    git(
        "remote",
        "set-url",
        "--push",
        "origin",
        re.sub(r".+@([^/]+)/", r"git@\1:", os.environ["CI_REPOSITORY_URL"]),
    )
    git("push", "-o", "ci.skip", "origin", get_version())


class SemverPart(str, Enum):
    patch = "patch"
    minor = "minor"
    major = "major"


@app.command()
def bump_version(semver_part: SemverPart = SemverPart.patch):
    git("fetch", "--tags", "-f")
    git("tag")
    try:
        v = _get_version()
        if semver_part.value == SemverPart.patch:
            n = semver.bump_patch(v)
        elif semver_part.value == SemverPart.minor:
            n = semver.bump_minor(v)
        elif semver_part.value == SemverPart.major:
            n = semver.bump_major(v)
    except (subprocess.CalledProcessError, ValueError):
        print("initialise versioning with 1.0.0")  # noqa
        git("tag", "1.0.0")
        return

    if "-" not in v:
        return

    print(f"bump from {v} to {n}")  # noqa
    data = toml.load("pyproject.toml")
    data["project"]["version"] = n
    with open("pyproject.toml", "w") as f:
        toml.dump(data, f)
    git("add", ".")
    git("commit", "--allow-empty", "-m", f"bump version to {n}")
    git("tag", "-a", n, "-m", f"release {n}")
    git("push", "--follow-tags")


@app.command()
def get_version():
    print(f"current version: {_get_version()}")  # noqa


@app.command()
def local():
    has_changes = git_num_changes() > 0
    try:
        if has_changes:
            git("add", ".")
            git("commit", "-m", "local debug commit")
        (
            lr["gitlab-ci-local"][
                "--privileged",
            ]
            & FG
        )
    except ProcessExecutionError:
        exit(1)
    finally:
        if has_changes:
            git("reset", "HEAD~1")



import glob
import json
import os
import subprocess

import typer
from plumbum import FG, ProcessExecutionError, local

app = typer.Typer()


def env_from_sourcing(source_path, include_unexported_variables=True):
    source = "{}source {}".format(
        "set -a && " if include_unexported_variables else "", source_path
    )
    dump = 'python3 -c "import os, json; print(json.dumps(dict(os.environ)))"'
    with subprocess.Popen(
        ["/bin/bash", "-c", f"{source} && {dump}"], stdout=subprocess.PIPE
    ) as pipe:
        return json.loads(pipe.stdout.read())


def _manage_env_vars():
    try:
        from dotenv import dotenv_values

        env_vars = dotenv_values(".env")
        for name, value in env_vars.items():
            local.env[name] = value
        return env_vars
    except ModuleNotFoundError:
        print("dotenv not found, dotenv autoloading disabled")  # noqa


def _run(*cmds):
    _manage_env_vars()
    print(cmds)  # noqa
    try:
        local.get(f".venv/bin/{cmds[0]}")[cmds[1:]] & FG
    except ProcessExecutionError:
        exit(1)


@app.command(
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def run(ctx: typer.Context):
    _run(*ctx.args)


@app.command()
def sh():
    _manage_env_vars()
    for k, v in env_from_sourcing(".venv/bin/activate").items():
        local.env[k] = v
    local.get(local.env["SHELL"].split("/")[-1]) & FG


@app.command()
def venv(requirements: str = typer.Argument("requirements.txt")):
    _manage_env_vars()
    local["python3"]["-m", "venv", ".venv"] & FG
    _run("pip", "install", "-U", "pip", "wheel", "gosu", ".")
    if os.path.isfile(requirements):
        _run("pip", "install", "-r", requirements)


def _get_pkg_pyfiles():
    files = list(
        filter(
            lambda e: all(
                [not (x in e) for x in ["node_modules", "migrations", "build"]]
            ),
            glob.glob("**/*.py", recursive=True),
        )
    )
    print(files)  # noqa
    return files


@app.command()
def fix():
    _run("pyupgrade", "--py38-plus", "--exit-zero-even-if-changed", *_get_pkg_pyfiles())
    _run("isort", "--profile", "black", *_get_pkg_pyfiles())
    _run("black", *_get_pkg_pyfiles())
    _run("flake8", "--config", f"{os.path.dirname(__file__)}/../.flake8")


@app.command()
def lint():
    _run("pyupgrade", "--py38-plus", *_get_pkg_pyfiles())
    _run("isort", "--profile", "black", "-c", *_get_pkg_pyfiles())
    _run("black", "--check", *_get_pkg_pyfiles())
    _run("flake8", "--config", f"{os.path.dirname(__file__)}/../.flake8")


@app.command()
def test():
    rcfile = f"--rcfile={os.path.dirname(__file__)}/../.coveragerc_python"
    local.env["PYTHONGPATH"] = "./src"
    _run(
        "coverage",
        "run",
        rcfile,
        "-m",
        "pytest",
        "-o",
        "console_output_style=progress",
    )
    _run("coverage", "report", rcfile)


@app.command()
def build():
    _run("flit", "build")


@app.command()
def precommit():
    fix()
    test()
    build()



import os

from gosu.cmds.ci import get_version, git_num_changes
from gosu.cmds.python import _get_pkg_pyfiles, _manage_env_vars, env_from_sourcing


def test_get_env_from_sourcing():
    e = env_from_sourcing(".venv/bin/activate")
    assert "PWD" in e


def test_get_env_vars():
    os.chdir("tests")
    e = _manage_env_vars()
    assert e["test"] == "1"


def test_get_pkg_pyfiles():
    e = _get_pkg_pyfiles()
    assert len(e) > 0


def test_get_version():
    assert get_version() != ""


def test_git_num_changes():
    assert git_num_changes() != ""



import os

import typer
from plumbum import local

from gosu.cmds.python import _manage_env_vars, _run, build, fix

app = typer.Typer()


def _pm(*cmds):
    _manage_env_vars()
    _run("python", "-m", f"{get_project()}.manage", *cmds)


def get_project():
    if "DJANGO_SETTINGS_MODULE" not in local.env:
        return "example"
    else:
        return local.env["DJANGO_SETTINGS_MODULE"].split(".")[0]


@app.command(
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def pm(ctx: typer.Context):
    _pm(*ctx.args)


@app.command()
def migrate():
    _pm("makemigrations")
    _pm("migrate")


@app.command()
def test():
    _manage_env_vars()

    local.env["DEFAULT_CACHE"] = "locmemcache://"
    local.env["QUEUE_CACHE"] = "locmemcache://"
    rcfile = f"--rcfile={os.path.dirname(__file__)}/../.coveragerc_django"
    _run(
        "coverage",
        "run",
        "--concurrency=multiprocessing",
        "--parallel-mode",
        rcfile,
        f"{get_project()}/manage.py",
        "test",
        "--parallel=3",
        ".",
        get_project(),
    )
    _run("coverage", "combine", rcfile)
    _run("coverage", "report", rcfile)


@app.command()
def precommit():
    fix()
    test()
    build()


@app.command()
def notebook():
    local.env["DJANGO_ALLOW_ASYNC_UNSAFE"] = True
    _pm("shell_plus", "--notebook")
